/*
 * This work is part of the White Rabbit project
 *
 * Released according to the GNU GPL, version 2 or any later version.
 */
#include <string.h>
#include <errno.h>

#include "shell.h"
#include "syscon.h"
#include "hw/memlayout.h"
#include "storage.h"
#include <flash.h>

/*
 * args[1] - where to write sdbfs image (0 - Flash, 1 - I2C EEPROM,
 *		2 - 1Wire EEPROM)
 * args[2] - base address for sdbfs image in Flash/EEPROM
 * args[3] - i2c address of EEPROM or blocksize of Flash
 */

static int cmd_sdb(const char *args[])
{
	uint8_t i2c_adr = FMC_EEPROM_ADR;
	int blocksize	= 1;
	uint8_t buf[1] = {0};
	uint32_t addr = 0;
	int count = 0;
	int i = 0;

	if (!args[0]) {
		sdb_print_devices();
		return 0;
	}
	if (!args[1] || !HAS_GENSDBFS)
		return -EINVAL;

	/* interpret args[3] as i2c adr or blocksize depending on memory type */
	if (args[3] && atoi(args[1]) == MEM_FLASH)
		blocksize = atoi(args[3])*1024;
	else if (args[3])
		i2c_adr = atoi(args[3]);

	/* Writing SDBFS image */
	if (!strcasecmp(args[0], "fs") && args[2]) {
		/* if all the parameters were specified from the cmd line, we
		 * use these */
		storage_gensdbfs(atoi(args[1]), atoi(args[2]), blocksize,
				i2c_adr);
		return 0;
	}
	if (!strcasecmp(args[0], "fs") && storage_cfg.valid &&
			atoi(args[1]) == MEM_FLASH) {
		/* if available, we can also use Flash parameters specified with
		 * HDL generics */
		storage_gensdbfs(MEM_FLASH, storage_cfg.baseadr,
				storage_cfg.blocksize, 0);
		return 0;
	}
	/* Erasing SDBFS image */
	if (!strcasecmp(args[0], "fse") && args[2]) {
		storage_sdbfs_erase(atoi(args[1]), atoi(args[2]), blocksize,
				i2c_adr);
		return 0;
	}
	if (!strcasecmp(args[0], "fse") && storage_cfg.valid &&
			atoi(args[1]) == MEM_FLASH) {
		storage_sdbfs_erase(MEM_FLASH, storage_cfg.baseadr,
				storage_cfg.blocksize, 0);
		return 0;
	}
	if (!strcasecmp(args[0], "wr") && atoi(args[1]) == MEM_FLASH) {
		addr   = atoi(args[2]);
		buf[0] = atoi(args[3]);
		flash_write(addr, buf, 1);
		pp_printf("written 0x%x at addr = 0x%x\n", buf[0], addr);
		flash_read(addr, buf, 1);
		pp_printf("read 0x%x from addr = 0x%x\n", buf[0], addr);
		return 0;
	}
	if (!strcasecmp(args[0], "fls") && atoi(args[1]) == MEM_FLASH) {
		buf[0] = flash_rsr();
		pp_printf("value of status reg 0x%x\n", buf[0]);
		return 0;
	}
	if (!strcasecmp(args[0], "rd") && atoi(args[1]) == MEM_FLASH) {
		addr   = atoi(args[2]);
		count  = atoi(args[3]);
		for (i = 0; i < count; i++) {
                     flash_read(addr+i, buf, 1);
		     pp_printf("0x%x: 0x%x\n", addr+i, buf[0]);
                }
		return 0;
	}
	if (!strcasecmp(args[0], "erase") && atoi(args[1]) == MEM_FLASH) {
		flash_erase(0, 0x7FFF);
		return 0;
	}
	return -EINVAL;
}

DEFINE_WRC_COMMAND(sdb) = {
	.name = "sdb",
	.exec = cmd_sdb,
};
